@@ -73,13 +73,11 @@ class AttestationCommunity(Community):
 
         self.request_cache = RequestCache()
 
-        self.decode_map.update({
-            chr(1): self.on_verify_attestation_request,
-            chr(2): self.on_attestation_chunk,
-            chr(3): self.on_challenge,
-            chr(4): self.on_challenge_response,
-            chr(5): self.on_request_attestation
-        })
+        self.add_message_handler(VerifyAttestationRequestPayload, self.on_verify_attestation_request)
+        self.add_message_handler(AttestationChunkPayload, self.on_attestation_chunk)
+        self.add_message_handler(ChallengePayload, self.on_challenge)
+        self.add_message_handler(ChallengeResponsePayload, self.on_challenge_response)
+        self.add_message_handler(RequestAttestationPayload, self.on_request_attestation)
 
     async def unload(self):
         await self.request_cache.shutdown()
@@ -428,7 +426,7 @@ class AttestationCommunity(Community):
                 proving_cache.attestation_callbacks(proving_cache.hash, proving_cache.relativity_map)
             else:
                 # Send another proving hash
-                honesty_check = algorithm.honesty_check and (ord(os.urandom(1)[0:1]) < 38)
+                honesty_check = algorithm.honesty_check and (os.urandom(1)[0] < 38)
                 honesty_check_byte = choice([0, 1, 2]) if honesty_check else -1
                 challenge = None
                 if honesty_check:
