@@ -71,7 +71,6 @@ default = {
                     'max_time': 10 * 60,
                     'max_time_inactive': 20,
                     'max_traffic': 250 * 1024 * 1024,
-                    'max_packets_without_reply': 50,
                     'dht_lookup_interval': 30
                 }
             },
@@ -94,32 +93,6 @@ default = {
             'initialize': {},
             'on_start': []
         },
-        {
-            'class': 'AttestationCommunity',
-            'key': "anonymous id",
-            'walkers': [{
-                'strategy': "RandomWalk",
-                'peers': 20,
-                'init': {
-                    'timeout': 3.0
-                }
-            }],
-            'initialize': {'anonymize': True},
-            'on_start': []
-        },
-        {
-            'class': 'IdentityCommunity',
-            'key': "anonymous id",
-            'walkers': [{
-                'strategy': "RandomWalk",
-                'peers': 20,
-                'init': {
-                    'timeout': 3.0
-                }
-            }],
-            'initialize': {'anonymize': True},
-            'on_start': []
-        },
         {
             'class': 'DHTDiscoveryCommunity',
             'key': "anonymous id",
