@@ -3,8 +3,10 @@ The tunnel community.
 
 Author(s): Egbert Bouman
 """
+import os
 from asyncio import iscoroutine, sleep
 from binascii import unhexlify
+from collections import defaultdict
 
 from .caches import *
 from .endpoint import TunnelEndpoint
@@ -19,30 +21,8 @@ from ...peer import Peer
 from ...requestcache import RequestCache
 from ...taskmanager import task
 
-message_to_payload = {
-    "data": (0, DataPayload),
-    "cell": (1, CellPayload),
-    "create": (2, CreatePayload),
-    "created": (3, CreatedPayload),
-    "extend": (4, ExtendPayload),
-    "extended": (5, ExtendedPayload),
-    "ping": (6, PingPayload),
-    "pong": (7, PongPayload),
-    "destroy": (10, DestroyPayload),
-    "establish-intro": (11, EstablishIntroPayload),
-    "intro-established": (12, IntroEstablishedPayload),
-    "establish-rendezvous": (15, EstablishRendezvousPayload),
-    "rendezvous-established": (16, RendezvousEstablishedPayload),
-    "create-e2e": (17, CreateE2EPayload),
-    "created-e2e": (18, CreatedE2EPayload),
-    "link-e2e": (19, LinkE2EPayload),
-    "linked-e2e": (20, LinkedE2EPayload),
-    "peers-request": (21, PeersRequestPayload),
-    "peers-response": (22, PeersResponsePayload)
-}
 
-
-def tc_lazy_wrapper_unsigned(*payloads):
+def unpack_cell(*payloads):
     """
     This function wrapper will unpack just the normal payloads for you, and handle a singular circuit_id parameter at
     the end of the parameter list
@@ -51,13 +31,11 @@ def tc_lazy_wrapper_unsigned(*payloads):
 
     ::
 
-        @tc_lazy_wrapper_unsigned(GlobalTimeDistributionPayload, IntroductionRequestPayload,
-                                  IntroductionResponsePayload, circuit_id)
-        def on_message(source_address, payload1, payload2):
+        @unpack_cell(DataPayload)
+        def on_message(source_address, payload):
             '''
             :type source_address: str
-            :type payload1: IntroductionRequestPayload
-            :type payload2: IntroductionResponsePayload
+            :type payload: DataPayload
             '''
             pass
     """
@@ -91,8 +69,6 @@ class TunnelSettings(object):
         self.max_time_inactive = 20
         self.max_traffic = 250 * 1024 * 1024
 
-        self.max_packets_without_reply = 50
-
         # Maximum number of seconds circuit creation is allowed to take. Within this time period, the unverified hop
         # of the circuit can still change in case it is unresponsive.
         self.circuit_timeout = 60
@@ -109,7 +85,7 @@ class TunnelSettings(object):
         self.remove_tunnel_delay = 5
 
         self.num_ip_circuits = 3
-        self.peer_flags = {PEER_FLAG_RELAY}
+        self.peer_flags = {PEER_FLAG_RELAY, PEER_FLAG_SPEED_TEST}
 
 
 class TunnelCommunity(Community):
@@ -130,23 +106,22 @@ class TunnelCommunity(Community):
         super(TunnelCommunity, self).__init__(*args, **kwargs)
 
         self.request_cache = RequestCache()
+        self.decode_map_private = {}
 
         # Messages that can arrive from the socket
-        self.decode_map.update({
-            chr(1): self.on_cell,
-            chr(10): self.on_destroy
-        })
+        self.add_message_handler(CellPayload, self.on_cell)
+        self.add_message_handler(DestroyPayload, self.on_destroy)
 
         # Messages that can arrive from a circuit (i.e., they are wrapped in a cell)
-        self.decode_map_private = {
-            chr(0): self.on_data,
-            chr(2): self.on_create,
-            chr(3): self.on_created,
-            chr(4): self.on_extend,
-            chr(5): self.on_extended,
-            chr(6): self.on_ping,
-            chr(7): self.on_pong
-        }
+        self.add_cell_handler(DataPayload, self.on_data)
+        self.add_cell_handler(CreatePayload, self.on_create)
+        self.add_cell_handler(CreatedPayload, self.on_created)
+        self.add_cell_handler(ExtendPayload, self.on_extend)
+        self.add_cell_handler(ExtendedPayload, self.on_extended)
+        self.add_cell_handler(PingPayload, self.on_ping)
+        self.add_cell_handler(PongPayload, self.on_pong)
+        self.add_cell_handler(TestRequestPayload, self.on_test_request)
+        self.add_cell_handler(TestResponsePayload, self.on_test_response)
 
         self.select_index = -1
         self.circuits = {}
@@ -160,7 +135,9 @@ class TunnelCommunity(Community):
 
         self.crypto = self.settings.crypto
 
-        self.logger.info("Setting exitnode = %s", PEER_FLAG_EXIT_ANY in self.settings.peer_flags)
+        self.logger.info("Exit settings: BT=%s, IPv8=%s",
+                         PEER_FLAG_EXIT_BT in self.settings.peer_flags,
+                         PEER_FLAG_EXIT_IPV8 in self.settings.peer_flags)
 
         self.crypto.initialize(self.my_peer.key)
 
@@ -184,6 +161,9 @@ class TunnelCommunity(Community):
 
         await super(TunnelCommunity, self).unload()
 
+    def add_cell_handler(self, payload_cls, handler):
+        self.decode_map_private[payload_cls.msg_id] = handler
+
     def _generate_circuit_id(self):
         circuit_id = random.getrandbits(32)
 
@@ -246,19 +226,19 @@ class TunnelCommunity(Community):
         for peer in list(self.candidates):
             if peer not in current_peers:
                 self.candidates.pop(peer)
-                self.logger.info("Removed candidate from candidates dictionary")
 
-    def get_candidates(self, flag):
+    def get_candidates(self, *requested_flags):
         return [peer for peer, flags in self.candidates.items()
-                if flag in flags and self.crypto.is_key_compatible(peer.public_key)]
+                if set(requested_flags) <= set(flags) and self.crypto.is_key_compatible(peer.public_key)]
 
     def get_max_time(self, circuit_id):
         return self.settings.max_time
 
-    def find_circuits(self, ctype=CIRCUIT_TYPE_DATA, state=CIRCUIT_STATE_READY, hops=None):
+    def find_circuits(self, ctype=CIRCUIT_TYPE_DATA, state=CIRCUIT_STATE_READY, exit_flags=None, hops=None):
         return [c for c in self.circuits.values()
                 if (state is None or c.state == state)
                 and (ctype is None or c.ctype == ctype)
+                and (exit_flags is None or set(exit_flags) <= set(c.exit_flags))
                 and (hops is None or hops == c.goal_hops)]
 
     def select_circuit(self, destination, hops):
@@ -269,31 +249,28 @@ class TunnelCommunity(Community):
         self.select_index = (self.select_index + 1) % len(circuits)
         return circuits[self.select_index]
 
-    def create_circuit(self, goal_hops, ctype=CIRCUIT_TYPE_DATA, required_exit=None, info_hash=None):
+    def create_circuit(self, goal_hops, ctype=CIRCUIT_TYPE_DATA, exit_flags=None, required_exit=None, info_hash=None):
         self.logger.info("Creating a new circuit of length %d (type: %s)", goal_hops, ctype)
-        exit_candidates = self.get_candidates(PEER_FLAG_EXIT_ANY)
-        if ctype == CIRCUIT_TYPE_IPV8:
-            exit_candidates = self.get_candidates(PEER_FLAG_EXIT_IPV8) or exit_candidates
-        relay_candidates = self.get_candidates(PEER_FLAG_RELAY)
 
         # Determine the last hop
         if not required_exit:
-            if ctype in [CIRCUIT_TYPE_DATA, CIRCUIT_TYPE_IPV8, CIRCUIT_TYPE_IP_SEEDER]:
-                required_exit = random.choice(exit_candidates) if exit_candidates else None
+            if exit_flags is not None:
+                exit_candidates = self.get_candidates(*exit_flags)
+            elif ctype == CIRCUIT_TYPE_DATA:
+                exit_candidates = self.get_candidates(PEER_FLAG_EXIT_BT)
+            elif ctype == CIRCUIT_TYPE_IP_SEEDER:
                 # For introduction points we prefer exit nodes, but perhaps a relay peer would also suffice..
-                if not required_exit and relay_candidates and ctype == CIRCUIT_TYPE_IP_SEEDER:
-                    required_exit = random.choice(relay_candidates)
+                exit_candidates = self.get_candidates(PEER_FLAG_EXIT_BT) or self.get_candidates(PEER_FLAG_RELAY)
             else:
                 # For exit nodes that don't exit actual data, we prefer relay candidates,
                 # but we also consider exit candidates.
-                if relay_candidates:
-                    required_exit = random.choice(relay_candidates)
-                elif exit_candidates:
-                    required_exit = random.choice(exit_candidates)
+                exit_candidates = self.get_candidates(PEER_FLAG_RELAY) or self.get_candidates(PEER_FLAG_EXIT_BT)
 
-        if not required_exit:
-            self.logger.info("Could not create circuit, no available exit-nodes")
-            return
+            if not exit_candidates:
+                self.logger.info("Could not create circuit, no available exit-nodes")
+                return
+
+            required_exit = random.choice(exit_candidates)
 
         # Determine the first hop
         if goal_hops == 1 and required_exit:
@@ -305,9 +282,9 @@ class TunnelCommunity(Community):
             # First build a list of hops, then filter the list. Avoids issues when create_circuit is called
             # from a different thread (caused by circuit.peer being reset to None).
             first_hops = [c.peer for c in self.circuits.values()]
-            first_hops = {h.address for h in first_hops if h}
-            possible_first_hops = [c for c in relay_candidates if c.address not in first_hops
-                                   and c.address != required_exit.address]
+            first_hops = {h for h in first_hops if h}
+            relay_candidates = self.get_candidates(PEER_FLAG_RELAY)
+            possible_first_hops = [c for c in relay_candidates if c not in first_hops and c != required_exit]
 
         if not possible_first_hops:
             self.logger.info("Could not create circuit, no first hop available")
@@ -328,8 +305,7 @@ class TunnelCommunity(Community):
         first_hop = random.choice(candidate_list)
         alt_first_hops = [c for c in candidate_list if c != first_hop]
 
-        circuit.unverified_hop = Hop(first_hop.public_key, flags=self.candidates.get(first_hop))
-        circuit.unverified_hop.address = first_hop.address
+        circuit.unverified_hop = Hop(first_hop, flags=self.candidates.get(first_hop))
         circuit.unverified_hop.dh_secret, circuit.unverified_hop.dh_first_part = self.crypto.generate_diffie_secret()
 
         self.logger.info("Adding first hop %s:%d to circuit %d", *(first_hop.address + (circuit.circuit_id,)))
@@ -337,11 +313,9 @@ class TunnelCommunity(Community):
         self.request_cache.add(RetryRequestCache(self, circuit, alt_first_hops, max_tries - 1,
                                                  self.send_initial_create, self.settings.next_hop_timeout))
 
-        self.increase_bytes_sent(circuit, self.send_cell([first_hop],
-                                                         "create",
-                                                         CreatePayload(circuit.circuit_id,
-                                                                       self.my_peer.public_key.key_to_bin(),
-                                                                       circuit.unverified_hop.dh_first_part)))
+        self.send_cell(first_hop, CreatePayload(circuit.circuit_id,
+                                                self.my_peer.public_key.key_to_bin(),
+                                                circuit.unverified_hop.dh_first_part))
 
     @task
     async def remove_circuit(self, circuit_id, additional_info='', remove_now=False, destroy=False):
@@ -456,46 +430,38 @@ class TunnelCommunity(Community):
         self.send_destroy(sock_addr, exit_socket.circuit_id, reason)
         self.logger.info("Destroy_exit_socket %s %s", exit_socket.circuit_id, sock_addr)
 
-    def is_relay(self, circuit_id):
-        return circuit_id > 0 and circuit_id in self.relay_from_to
-
-    def is_circuit(self, circuit_id):
-        return circuit_id > 0 and circuit_id in self.circuits
-
-    def is_exit(self, circuit_id):
-        return circuit_id > 0 and circuit_id in self.exit_sockets
-
-    def send_cell(self, candidates, message_type, payload, circuit_id=None):
-        message_id, _ = message_to_payload[message_type]
-        circuit_id = circuit_id or payload.circuit_id
-        if isinstance(payload, DataPayload):
+    def send_cell(self, peer, payload):
+        circuit_id = payload.circuit_id
+        if isinstance(payload, (DataPayload, TestRequestPayload, TestResponsePayload)):
             message = payload.to_bin()
         else:
             message = self.serializer.pack_multiple(payload.to_pack_list()[1:])[0]
-        cell = CellPayload(circuit_id, pack('!B', message_id) + message, message_id in NO_CRYPTO_PACKETS)
+        cell = CellPayload(circuit_id, pack('!B', payload.msg_id) + message, payload.msg_id in NO_CRYPTO_PACKETS)
         try:
             cell.encrypt(self.crypto, self.circuits.get(circuit_id), self.relay_session_keys.get(circuit_id))
         except CryptoException as e:
             self.logger.warning(str(e))
             return
         packet = cell.to_bin(self._prefix)
-        return self.send_packet(candidates, packet)
+        packet_len = self.send_packet(peer, packet)
 
-    def send_data(self, candidates, circuit_id, dest_address, source_address, data):
+        tunnel_obj = self.circuits.get(circuit_id) or self.relay_from_to.get(circuit_id)
+        if tunnel_obj:
+            tunnel_obj.bytes_up += packet_len
+
+    def send_data(self, peer, circuit_id, dest_address, source_address, data):
         payload = DataPayload(circuit_id, dest_address, source_address, data)
-        return self.send_cell(candidates, "data", payload, circuit_id)
+        return self.send_cell(peer, payload)
 
-    def send_packet(self, candidates, packet):
-        for candidate in candidates:
-            address = candidate if isinstance(candidate, tuple) else candidate.address
-            self.endpoint.send(address, packet)
+    def send_packet(self, peer, packet):
+        self.endpoint.send(peer if isinstance(peer, tuple) else peer.address, packet)
         return len(packet)
 
-    def send_destroy(self, candidate, circuit_id, reason):
+    def send_destroy(self, peer, circuit_id, reason):
         auth = BinMemberAuthenticationPayload(self.my_peer.public_key.key_to_bin()).to_pack_list()
         payload = DestroyPayload(circuit_id, reason).to_pack_list()
         packet = self._ez_pack(self._prefix, 10, [auth, payload])
-        self.send_packet([candidate], packet)
+        self.send_packet(peer, packet)
 
     def relay_cell(self, cell):
         next_relay = self.relay_from_to[cell.circuit_id]
@@ -519,7 +485,7 @@ class TunnelCommunity(Community):
 
         cell.circuit_id = next_relay.circuit_id
         packet = cell.to_bin(self._prefix)
-        self.increase_bytes_sent(next_relay, self.send_packet([next_relay.peer], packet))
+        next_relay.bytes_up += self.send_packet(next_relay.peer, packet)
 
     def _ours_on_created_extended(self, circuit, payload):
         hop = circuit.unverified_hop
@@ -530,7 +496,7 @@ class TunnelCommunity(Community):
             hop.session_keys = self.crypto.generate_session_keys(shared_secret)
 
         except CryptoException:
-            self.remove_circuit(circuit.circuit_id, "error while verifying shared secret, bailing out.")
+            self.remove_circuit(circuit.circuit_id, "error while verifying shared secret")
             return
 
         circuit.unverified_hop = None
@@ -577,7 +543,7 @@ class TunnelCommunity(Community):
 
         if extend_hop_public_bin:
             extend_hop_public_key = self.crypto.key_from_public_bin(extend_hop_public_bin)
-            circuit.unverified_hop = Hop(extend_hop_public_key,
+            circuit.unverified_hop = Hop(Peer(extend_hop_public_key),
                                          flags=self.candidates.get(Peer(extend_hop_public_bin)))
             circuit.unverified_hop.dh_secret, circuit.unverified_hop.dh_first_part = \
                 self.crypto.generate_diffie_secret()
@@ -592,15 +558,13 @@ class TunnelCommunity(Community):
             self.request_cache.add(RetryRequestCache(self, circuit, alt_candidates, max_tries - 1,
                                                      self.send_extend, self.settings.next_hop_timeout))
 
-            self.increase_bytes_sent(circuit, self.send_cell([circuit.peer],
-                                                             "extend",
-                                                             ExtendPayload(circuit.circuit_id,
-                                                                           circuit.unverified_hop.node_public_key,
-                                                                           extend_hop_addr,
-                                                                           circuit.unverified_hop.dh_first_part)))
+            self.send_cell(circuit.peer, ExtendPayload(circuit.circuit_id,
+                                                       circuit.unverified_hop.node_public_key,
+                                                       extend_hop_addr,
+                                                       circuit.unverified_hop.dh_first_part))
 
         else:
-            self.remove_circuit(circuit.circuit_id, "no candidates to extend, bailing out.")
+            self.remove_circuit(circuit.circuit_id, "no candidates to extend")
 
     def extract_peer_flags(self, extra_bytes):
         if not extra_bytes:
@@ -631,12 +595,12 @@ class TunnelCommunity(Community):
         cell = CellPayload.from_bin(data)
         circuit_id = cell.circuit_id
 
-        if self.is_relay(circuit_id):
+        if circuit_id in self.relay_from_to:
             next_relay = self.relay_from_to[circuit_id]
             this_relay = self.relay_from_to.get(next_relay.circuit_id, None)
             if this_relay:
                 this_relay.beat_heart()
-                self.increase_bytes_received(this_relay, len(data))
+                this_relay.bytes_down += len(data)
             self.logger.debug("Relaying cell from circuit %d to %d", circuit_id, next_relay.circuit_id)
             self.relay_cell(cell)
             return
@@ -644,25 +608,26 @@ class TunnelCommunity(Community):
         circuit = self.circuits.get(circuit_id, None)
         try:
             cell.decrypt(self.crypto, circuit=circuit, relay_session_keys=self.relay_session_keys.get(circuit_id))
-        except CryptoException:
+        except CryptoException as e:
+            self.logger.debug(str(e))
             if circuit:
                 self.send_destroy(circuit.peer, circuit_id, 0)
             return
         self.logger.debug("Got cell(%s) from circuit %d (sender %s, receiver %s)",
                           cell.message[0], circuit_id, source_address, self.my_peer)
-        if cell.plaintext and ord(cell.message[0:1]) not in NO_CRYPTO_PACKETS:
+        if cell.plaintext and cell.message[0] not in NO_CRYPTO_PACKETS:
             self.logger.warning('Dropping cell (only create/created can have plaintext flag set)')
             return
         self.on_packet_from_circuit(source_address, cell.unwrap(self._prefix), circuit_id)
 
         if circuit:
             circuit.beat_heart()
-            self.increase_bytes_received(circuit, len(data))
+            circuit.bytes_down += len(data)
 
     def on_packet_from_circuit(self, source_address, data, circuit_id):
         if self._prefix != data[:22]:
             return
-        msg_id = chr(ord(data[22:23]))
+        msg_id = data[22]
         if msg_id in self.decode_map_private:
             try:
                 handler = self.decode_map_private[msg_id]
@@ -696,7 +661,7 @@ class TunnelCommunity(Community):
         self.relay_session_keys[circuit_id] = self.crypto.generate_session_keys(shared_secret)
 
         peers_list = [peer for peer in self.get_candidates(PEER_FLAG_RELAY)
-                      if peer not in self.get_candidates(PEER_FLAG_EXIT_ANY)][:4]
+                      if peer not in self.get_candidates(PEER_FLAG_EXIT_BT)][:4]
         peers_keys = {c.public_key.key_to_bin(): c for c in peers_list}
 
         peer = Peer(create_payload.node_public_key, previous_node_address)
@@ -706,10 +671,10 @@ class TunnelCommunity(Community):
         candidate_list_enc = self.crypto.encrypt_str(encode(list(peers_keys.keys())),
                                                      *self.crypto.get_session_keys(self.relay_session_keys[circuit_id],
                                                                                    EXIT_NODE))
-        self.send_cell([Peer(create_payload.node_public_key, previous_node_address)], "created",
+        self.send_cell(Peer(create_payload.node_public_key, previous_node_address),
                        CreatedPayload(circuit_id, key, auth, candidate_list_enc))
 
-    @tc_lazy_wrapper_unsigned(CreatePayload)
+    @unpack_cell(CreatePayload)
     async def on_create(self, source_address, payload, _):
         if not self.settings.peer_flags:
             self.logger.warning("Ignoring create for circuit %d", payload.circuit_id)
@@ -724,7 +689,7 @@ class TunnelCommunity(Community):
         else:
             self.logger.warning("We're not joining circuit with ID %s", payload.circuit_id)
 
-    @tc_lazy_wrapper_unsigned(CreatedPayload)
+    @unpack_cell(CreatedPayload)
     def on_created(self, source_address, payload, _):
         circuit_id = payload.circuit_id
         self.directions[circuit_id] = ORIGINATOR
@@ -741,17 +706,15 @@ class TunnelCommunity(Community):
             self.directions[request.from_circuit_id] = EXIT_NODE
             self.remove_exit_socket(request.from_circuit_id)
 
-            self.send_cell([relay.peer], "extended", ExtendedPayload(relay.circuit_id,
-                                                                     payload.key,
-                                                                     payload.auth,
-                                                                     payload.candidate_list_enc))
+            self.send_cell(relay.peer,
+                           ExtendedPayload(relay.circuit_id, payload.key, payload.auth, payload.candidate_list_enc))
         elif self.request_cache.has("retry", payload.circuit_id):
             circuit = self.circuits[circuit_id]
             self._ours_on_created_extended(circuit, payload)
         else:
             self.logger.warning("Received unexpected created for circuit %d", payload.circuit_id)
 
-    @tc_lazy_wrapper_unsigned(ExtendPayload)
+    @unpack_cell(ExtendPayload)
     async def on_extend(self, source_address, payload, _):
         if PEER_FLAG_RELAY not in self.settings.peer_flags:
             self.logger.warning("Ignoring create for circuit %d", payload.circuit_id)
@@ -795,13 +758,12 @@ class TunnelCommunity(Community):
 
         self.logger.info("Extending circuit, got candidate with IP %s:%d from cache", *extend_candidate.address)
 
-        self.request_cache.add(CreateRequestCache(self, to_circuit_id, circuit_id,
-                                                  candidate, extend_candidate))
+        self.request_cache.add(CreateRequestCache(self, to_circuit_id, circuit_id, candidate, extend_candidate))
 
-        self.send_cell([extend_candidate], "create",
+        self.send_cell(extend_candidate,
                        CreatePayload(to_circuit_id, self.my_peer.public_key.key_to_bin(), payload.key))
 
-    @tc_lazy_wrapper_unsigned(ExtendedPayload)
+    @unpack_cell(ExtendedPayload)
     def on_extended(self, source_address, payload, _):
         if not self.request_cache.has("retry", payload.circuit_id):
             self.logger.warning("Received unexpected extended for circuit %s", payload.circuit_id)
@@ -857,7 +819,7 @@ class TunnelCommunity(Community):
             else:
                 self.logger.warning("Cannot exit data, destination is 0.0.0.0:0")
 
-    @tc_lazy_wrapper_unsigned(PingPayload)
+    @unpack_cell(PingPayload)
     def on_ping(self, source_address, payload, _):
         if not (payload.circuit_id in self.circuits
                 or payload.circuit_id in self.exit_sockets
@@ -868,10 +830,10 @@ class TunnelCommunity(Community):
         if exit_socket:
             exit_socket.beat_heart()
 
-        self.send_cell([source_address], "pong", PongPayload(payload.circuit_id, payload.identifier))
+        self.send_cell(source_address, PongPayload(payload.circuit_id, payload.identifier))
         self.logger.debug("Got ping from %s", source_address)
 
-    @tc_lazy_wrapper_unsigned(PongPayload)
+    @unpack_cell(PongPayload)
     def on_pong(self, source_address, payload, _):
         if not self.request_cache.has("ping", payload.identifier):
             self.logger.warning("Invalid ping circuit_id")
@@ -887,9 +849,9 @@ class TunnelCommunity(Community):
             if circuit.state in [CIRCUIT_STATE_READY, CIRCUIT_STATE_EXTENDING] \
                     and circuit.circuit_id not in exclude \
                     and circuit.hops:
-                cache = self.request_cache.add(PingRequestCache(self, circuit))
-                self.increase_bytes_sent(circuit, self.send_cell([circuit.peer], "ping",
-                                                                 PingPayload(circuit.circuit_id, cache.number)))
+                cache = PingRequestCache(self, circuit)
+                self.request_cache.add(cache)
+                self.send_cell(circuit.peer, PingPayload(circuit.circuit_id, cache.number))
 
     @lazy_wrapper(DestroyPayload)
     def on_destroy(self, peer, payload):
@@ -913,40 +875,63 @@ class TunnelCommunity(Community):
             self.logger.warning("Invalid or unauthorized destroy")
 
     def exit_data(self, circuit_id, sock_addr, destination, data):
-        is_ipv8 = DataChecker.could_be_ipv8(data)
-        is_ipv8_tunnel = is_ipv8 and self._prefix == data[:22]
-
-        if not (is_ipv8 or PEER_FLAG_EXIT_ANY in self.settings.peer_flags):
-            self.logger.error("Dropping data packets, refusing to be an exit node for data")
-
-        elif not (is_ipv8_tunnel
-                  or PEER_FLAG_EXIT_IPV8 in self.settings.peer_flags
-                  or PEER_FLAG_EXIT_ANY in self.settings.peer_flags):
-            self.logger.error("Dropping data packets, refusing to be an exit node for ipv8")
-
-        elif circuit_id in self.exit_sockets:
-            if not self.exit_sockets[circuit_id].enabled:
-                # Check that we got the data from the correct IP.
-                if sock_addr[0] == self.exit_sockets[circuit_id].peer.address[0]:
-                    self.exit_sockets[circuit_id].enable()
-                else:
-                    self.logger.error("Dropping outbound relayed packet: IP's are %s != %s",
-                                      str(sock_addr), str(self.exit_sockets[circuit_id].peer.address))
-            try:
-                self.exit_sockets[circuit_id].sendto(data, destination)
-            except Exception:
-                self.logger.warning("Dropping data packets while EXITing")
-        else:
+        if circuit_id not in self.exit_sockets:
             self.logger.error("Dropping data packets with unknown circuit_id")
+            return
 
-    def increase_bytes_sent(self, obj, num_bytes):
-        obj.bytes_up += num_bytes
-
-    def increase_bytes_received(self, obj, num_bytes):
-        obj.bytes_down += num_bytes
+        if not self.exit_sockets[circuit_id].enabled:
+            # Check that we got the data from the correct IP.
+            if sock_addr[0] == self.exit_sockets[circuit_id].peer.address[0]:
+                self.exit_sockets[circuit_id].enable()
+            else:
+                self.logger.error("Dropping outbound relayed packet: IP's are %s != %s",
+                                  str(sock_addr), str(self.exit_sockets[circuit_id].peer.address))
+                return
+        try:
+            self.exit_sockets[circuit_id].sendto(data, destination)
+        except Exception:
+            self.logger.warning("Dropping data packets while exiting")
 
     async def dht_peer_lookup(self, mid, peer=None):
         if self.dht_provider:
             await self.dht_provider.peer_lookup(mid, peer)
         else:
             self.logger.error("Need a DHT provider to connect to a peer using the DHT")
+
+    def send_test_request(self, circuit, request_size=0, response_size=0):
+        cache = TestRequestCache(self, circuit)
+        self.request_cache.add(cache)
+        self.send_cell(circuit.peer,
+                       TestRequestPayload(circuit.circuit_id, cache.number, response_size, os.urandom(request_size)))
+        return cache.future
+
+    def on_test_request(self, source_address, data, circuit_id):
+        if PEER_FLAG_SPEED_TEST not in self.settings.peer_flags:
+            self.logger.warning("Ignoring test-request from circuit %d", circuit_id)
+            return
+
+        payload = TestRequestPayload.from_bin(data)
+        exit_socket = self.exit_sockets.get(circuit_id)
+        circuit = self.circuits.get(circuit_id)
+        if not exit_socket and not (circuit and circuit.ctype == CIRCUIT_TYPE_RP_SEEDER):
+            self.logger.error("Dropping test-request with unknown circuit_id")
+            return
+
+        self.logger.debug("Got test-request (%d) from %s, replying with response", circuit_id, source_address)
+        (exit_socket or circuit).beat_heart()
+        self.send_cell(source_address,
+                       TestResponsePayload(circuit_id, payload.identifier, os.urandom(payload.response_size)))
+
+    def on_test_response(self, source_address, data, circuit_id):
+        payload = TestResponsePayload.from_bin(data)
+        circuit = self.circuits.get(circuit_id)
+        if not circuit:
+            self.logger.error("Dropping test-response with unknown circuit_id")
+            return
+        if not self.request_cache.has("test-request", payload.identifier):
+            self.logger.error("Dropping unexpected test-response")
+            return
+
+        self.logger.debug("Got test-response (%d) from %s", circuit_id, source_address)
+        cache = self.request_cache.pop("test-request", payload.identifier)
+        cache.future.set_result((payload.data, time.time() - cache.ts))
