@@ -7,11 +7,10 @@ import binascii
 import os
 import random
 import struct
-import time
 from asyncio import gather, iscoroutine
 
 from .caches import *
-from .community import TunnelCommunity, message_to_payload, tc_lazy_wrapper_unsigned
+from .community import TunnelCommunity, unpack_cell
 from .payload import *
 from .tunnel import (CIRCUIT_ID_PORT, CIRCUIT_TYPE_IP_SEEDER, CIRCUIT_TYPE_RP_DOWNLOADER, CIRCUIT_TYPE_RP_SEEDER,
                      DESTROY_REASON_LEAVE_SWARM, DESTROY_REASON_UNNEEDED, EXIT_NODE, EXIT_NODE_SALT, Hop,
@@ -42,25 +41,21 @@ class HiddenTunnelCommunity(TunnelCommunity):
         self.rendezvous_point_for = {}  # {cookie: TunnelExitSocket}
 
         # Messages that can arrive from the socket
-        self.decode_map.update({
-            chr(17): self.on_create_e2e,
-            chr(21): self.on_peers_request,
-            chr(22): self.on_peers_response
-        })
+        self.add_message_handler(CreateE2EPayload, self.on_create_e2e)
+        self.add_message_handler(PeersRequestPayload, self.on_peers_request)
+        self.add_message_handler(PeersResponsePayload, self.on_peers_response)
 
         # Messages that can arrive from a circuit (i.e., they are wrapped in a cell)
-        self.decode_map_private.update({
-            chr(11): self.on_establish_intro,
-            chr(12): self.on_intro_established,
-            chr(15): self.on_establish_rendezvous,
-            chr(16): self.on_rendezvous_established,
-            chr(17): self.on_create_e2e,
-            chr(18): self.on_created_e2e,
-            chr(19): self.on_link_e2e,
-            chr(20): self.on_linked_e2e,
-            chr(21): self.on_peers_request,
-            chr(22): self.on_peers_response
-        })
+        self.add_cell_handler(EstablishIntroPayload, self.on_establish_intro)
+        self.add_cell_handler(IntroEstablishedPayload, self.on_intro_established)
+        self.add_cell_handler(EstablishRendezvousPayload, self.on_establish_rendezvous)
+        self.add_cell_handler(RendezvousEstablishedPayload, self.on_rendezvous_established)
+        self.add_cell_handler(CreateE2EPayload, self.on_create_e2e)
+        self.add_cell_handler(CreatedE2EPayload, self.on_created_e2e)
+        self.add_cell_handler(LinkE2EPayload, self.on_link_e2e)
+        self.add_cell_handler(LinkedE2EPayload, self.on_linked_e2e)
+        self.add_cell_handler(PeersRequestPayload, self.on_peers_request)
+        self.add_cell_handler(PeersResponsePayload, self.on_peers_response)
 
         self.register_task("do_peer_discovery", self.do_peer_discovery, interval=10)
 
@@ -83,7 +78,7 @@ class HiddenTunnelCommunity(TunnelCommunity):
             self.leave_swarm(info_hash)
 
         self.swarms[info_hash] = Swarm(info_hash, hops, self.send_peers_request,
-                                       self.crypto.generate_key(u"curve25519") if seeding else None)
+                                       self.crypto.generate_key("curve25519") if seeding else None)
         self.e2e_callbacks[info_hash] = callback
 
     def leave_swarm(self, info_hash):
@@ -247,16 +242,15 @@ class HiddenTunnelCommunity(TunnelCommunity):
             return self.settings.max_time_ip
         return super(HiddenTunnelCommunity, self).get_max_time(circuit_id)
 
-    def tunnel_data(self, circuit, destination, message_type, payload):
-        message_id, _ = message_to_payload[message_type]
-        packet = self._ez_pack(self._prefix, message_id, [payload.to_pack_list()], False)
+    def tunnel_data(self, circuit, destination, payload):
+        packet = self._ez_pack(self._prefix, payload.msg_id, [payload.to_pack_list()], False)
         pre = ('0.0.0.0', 0)
         post = ('0.0.0.0', 0)
         if isinstance(circuit, TunnelExitSocket):
             post = destination
         else:
             pre = destination
-        self.send_data([circuit.peer], circuit.circuit_id, pre, post, packet)
+        self.send_data(circuit.peer, circuit.circuit_id, pre, post, packet)
 
     def select_circuit(self, destination, hops):
         # Make sure that we select the right circuit when dealing with an e2e connection
@@ -276,20 +270,21 @@ class HiddenTunnelCommunity(TunnelCommunity):
             return fail(RuntimeError("No circuit for peers-request"))
 
         # Send a peers-request message over this circuit
-        cache = self.request_cache.add(PeersRequestCache(self, circuit, info_hash))
+        cache = PeersRequestCache(self, circuit, info_hash)
+        self.request_cache.add(cache)
         payload = PeersRequestPayload(circuit.circuit_id, cache.number, info_hash)
 
         # Ask an introduction point if available (in which case we'll use PEX), otherwise let
         # the exit node do a DHT request.
         if target and target.peer.public_key != circuit.hops[-1].public_key:
-            self.tunnel_data(circuit, target.peer.address, u"peers-request", payload)
+            self.tunnel_data(circuit, target.peer.address, payload)
             self.logger.info("Sending peers request (intro point %s)", target.peer)
         else:
-            self.send_cell([circuit.peer], u"peers-request", payload)
+            self.send_cell(circuit.peer, payload)
             self.logger.info("Sending peers request as cell")
         return cache.future
 
-    @tc_lazy_wrapper_unsigned(PeersRequestPayload)
+    @unpack_cell(PeersRequestPayload)
     async def on_peers_request(self, source_address, payload, circuit_id=None):
         info_hash = payload.info_hash
         self.logger.info("Doing hidden seeders lookup for info_hash %s", binascii.hexlify(info_hash))
@@ -313,19 +308,18 @@ class HiddenTunnelCommunity(TunnelCommunity):
 
         if circuit_id is not None:
             # Send back to origin
-            self.send_cell([target_addr], 'peers-response', payload)
+            self.send_cell(target_addr, payload)
         else:
             # Send back to exit node
-            message_id, _ = message_to_payload['peers-response']
-            packet = self._ez_pack(self._prefix, message_id, [payload.to_pack_list()], False)
-            self.send_packet([target_addr], packet)
+            packet = self._ez_pack(self._prefix, payload.msg_id, [payload.to_pack_list()], False)
+            self.send_packet(target_addr, packet)
 
-    @tc_lazy_wrapper_unsigned(PeersResponsePayload)
+    @unpack_cell(PeersResponsePayload)
     def on_peers_response(self, source_address, payload, circuit_id):
-        if not self.request_cache.has(u"peers-request", payload.identifier):
+        if not self.request_cache.has("peers-request", payload.identifier):
             self.logger.warning('Got a peers-response with an unknown identifier')
             return
-        cache = self.request_cache.pop(u"peers-request", payload.identifier)
+        cache = self.request_cache.pop("peers-request", payload.identifier)
 
         _, peers = decode(payload.peers)
         self.logger.info("Received peers-response containing %d peers", len(peers))
@@ -339,21 +333,22 @@ class HiddenTunnelCommunity(TunnelCommunity):
             self.logger.error("No circuit for contacting the introduction point")
             return
 
-        hop = Hop(LibNaCLPK(intro_point.seeder_pk[10:]))
+        hop = Hop(Peer(LibNaCLPK(intro_point.seeder_pk[10:])))
         hop.dh_secret, hop.dh_first_part = self.crypto.generate_diffie_secret()
         self.logger.info('Creating e2e circuit for introduction point %s', intro_point.peer)
-        cache = self.request_cache.add(E2ERequestCache(self, info_hash, hop, intro_point))
-        self.tunnel_data(circuit, intro_point.peer.address, u'create-e2e',
+        cache = E2ERequestCache(self, info_hash, hop, intro_point)
+        self.request_cache.add(cache)
+        self.tunnel_data(circuit, intro_point.peer.address,
                          CreateE2EPayload(cache.number, info_hash, hop.node_public_key, hop.dh_first_part))
 
-    @tc_lazy_wrapper_unsigned(CreateE2EPayload)
+    @unpack_cell(CreateE2EPayload)
     async def on_create_e2e(self, source_address, payload, circuit_id=None):
         # If we have received this message over a socket, we need to forward it
         if circuit_id is None:
             if payload.node_public_key in self.intro_point_for:
                 self.logger.info('On create-e2e: forwarding message because received over socket')
                 relay_circuit, _ = self.intro_point_for[payload.node_public_key]
-                self.tunnel_data(relay_circuit, source_address, u'create-e2e', payload)
+                self.tunnel_data(relay_circuit, source_address, payload)
             else:
                 self.logger.info('On create-e2e: dropping message for unknown seeder key %s',
                                  binascii.hexlify(payload.node_public_key))
@@ -374,16 +369,16 @@ class HiddenTunnelCommunity(TunnelCommunity):
             *self.crypto.get_session_keys(rendezvous_point.circuit.hs_session_keys, EXIT_NODE))
 
         circuit = self.circuits[circuit_id]
-        self.tunnel_data(circuit, source_address, u'created-e2e',
+        self.tunnel_data(circuit, source_address,
                          CreatedE2EPayload(payload.identifier, Y, AUTH, rp_info_enc))
 
-    @tc_lazy_wrapper_unsigned(CreatedE2EPayload)
+    @unpack_cell(CreatedE2EPayload)
     async def on_created_e2e(self, source_address, payload, circuit_id):
-        if not self.request_cache.has(u"e2e-request", payload.identifier):
+        if not self.request_cache.has("e2e-request", payload.identifier):
             self.logger.warning("Invalid created-e2e identifier")
             return
 
-        cache = self.request_cache.pop(u"e2e-request", payload.identifier)
+        cache = self.request_cache.pop("e2e-request", payload.identifier)
         shared_secret = self.crypto.verify_and_generate_shared_secret(cache.hop.dh_secret,
                                                                       payload.key,
                                                                       payload.auth,
@@ -401,10 +396,11 @@ class HiddenTunnelCommunity(TunnelCommunity):
         if circuit:
             self.swarms[cache.info_hash].add_connection(circuit, cache.intro_point)
             if await circuit.ready:
-                cache = self.request_cache.add(LinkRequestCache(self, circuit, cache.info_hash, session_keys))
-                self.send_cell([circuit.peer], u'link-e2e', LinkE2EPayload(circuit.circuit_id, cache.number, cookie))
+                cache = LinkRequestCache(self, circuit, cache.info_hash, session_keys)
+                self.request_cache.add(cache)
+                self.send_cell(circuit.peer, LinkE2EPayload(circuit.circuit_id, cache.number, cookie))
 
-    @tc_lazy_wrapper_unsigned(LinkE2EPayload)
+    @unpack_cell(LinkE2EPayload)
     def on_link_e2e(self, source_address, payload, circuit_id):
         if payload.cookie not in self.rendezvous_point_for:
             self.logger.warning("Not a rendezvous point for this cookie")
@@ -426,15 +422,15 @@ class HiddenTunnelCommunity(TunnelCommunity):
         self.relay_from_to[circuit.circuit_id] = RelayRoute(relay_circuit.circuit_id, relay_circuit.peer, True)
         self.relay_from_to[relay_circuit.circuit_id] = RelayRoute(circuit.circuit_id, circuit.peer, True)
 
-        self.send_cell([source_address], u"linked-e2e", LinkedE2EPayload(circuit.circuit_id, payload.identifier))
+        self.send_cell(source_address, LinkedE2EPayload(circuit.circuit_id, payload.identifier))
 
-    @tc_lazy_wrapper_unsigned(LinkedE2EPayload)
+    @unpack_cell(LinkedE2EPayload)
     def on_linked_e2e(self, source_address, payload, circuit_id):
-        if not self.request_cache.has(u"link-request", payload.identifier):
+        if not self.request_cache.has("link-request", payload.identifier):
             self.logger.warning("Invalid linked-e2e identifier")
             return
 
-        cache = self.request_cache.pop(u"link-request", payload.identifier)
+        cache = self.request_cache.pop("link-request", payload.identifier)
         circuit = cache.circuit
         circuit.e2e = True
         circuit.hs_session_keys = cache.hs_session_keys
@@ -462,12 +458,12 @@ class HiddenTunnelCommunity(TunnelCommunity):
             # We got a circuit, now let's create an introduction point
             seed_pk = self.swarms[info_hash].seeder_sk.pub().key_to_bin()
             circuit_id = circuit.circuit_id
-            cache = self.request_cache.add(IPRequestCache(self, circuit))
-            self.send_cell([circuit.peer], 'establish-intro', EstablishIntroPayload(circuit_id, cache.number,
-                                                                                    info_hash, seed_pk))
+            cache = IPRequestCache(self, circuit)
+            self.request_cache.add(cache)
+            self.send_cell(circuit.peer, EstablishIntroPayload(circuit_id, cache.number, info_hash, seed_pk))
             self.logger.info("Established introduction tunnel %s", circuit_id)
 
-    @tc_lazy_wrapper_unsigned(EstablishIntroPayload)
+    @unpack_cell(EstablishIntroPayload)
     def on_establish_intro(self, source_address, payload, circuit_id):
         if payload.public_key in self.intro_point_for:
             self.logger.warning('Already have an introduction point for %s', binascii.hexlify(payload.public_key))
@@ -482,10 +478,9 @@ class HiddenTunnelCommunity(TunnelCommunity):
             self.logger.error('No IPv8 service object available, cannot start PEXCommunity')
         elif payload.info_hash not in self.pex:
             community = PexCommunity(self.my_peer, self.endpoint, Network(), info_hash=payload.info_hash)
-            self.ipv8.overlays.append(community)
             # Since IPv8 takes a step every .5s until we have 10 peers, the PexCommunity will generate
             # a lot of traffic in case there are <10 peers in existence. Therefore, we slow the walk down to a 5s/step.
-            self.ipv8.strategies.append((RandomWalk(community, target_interval=5), 10))
+            self.ipv8.add_strategy(community, RandomWalk(community, target_interval=5), 10)
             self.pex[payload.info_hash] = community
 
         # PEX announce
@@ -496,16 +491,15 @@ class HiddenTunnelCommunity(TunnelCommunity):
         self.dht_announce(payload.info_hash, IntroductionPoint(Peer(self.my_peer.key, self.my_estimated_wan),
                                                                payload.public_key))
 
-        self.send_cell([source_address], u"intro-established",
-                       IntroEstablishedPayload(circuit.circuit_id, payload.identifier))
+        self.send_cell(source_address, IntroEstablishedPayload(circuit.circuit_id, payload.identifier))
 
-    @tc_lazy_wrapper_unsigned(IntroEstablishedPayload)
+    @unpack_cell(IntroEstablishedPayload)
     def on_intro_established(self, source_address, payload, circuit_id):
-        if not self.request_cache.has(u"establish-intro", payload.identifier):
+        if not self.request_cache.has("establish-intro", payload.identifier):
             self.logger.warning("Invalid intro-established request identifier")
             return
 
-        self.request_cache.pop(u"establish-intro", payload.identifier)
+        self.request_cache.pop("establish-intro", payload.identifier)
         self.logger.info("Got intro-established from %s", source_address)
 
     async def create_rendezvous_point(self, info_hash):
@@ -515,27 +509,26 @@ class HiddenTunnelCommunity(TunnelCommunity):
         if circuit and await circuit.ready:
             # We got a circuit, now let's create a rendezvous point
             rp = RendezvousPoint(circuit, os.urandom(20))
-            cache = self.request_cache.add(RPRequestCache(self, rp))
-            self.send_cell([circuit.peer],
-                           'establish-rendezvous', EstablishRendezvousPayload(circuit.circuit_id,
-                                                                              cache.number, rp.cookie))
+            cache = RPRequestCache(self, rp)
+            self.request_cache.add(cache)
+            self.send_cell(circuit.peer, EstablishRendezvousPayload(circuit.circuit_id, cache.number, rp.cookie))
             return rp
 
-    @tc_lazy_wrapper_unsigned(EstablishRendezvousPayload)
+    @unpack_cell(EstablishRendezvousPayload)
     def on_establish_rendezvous(self, source_address, payload, circuit_id):
         circuit = self.exit_sockets[circuit_id]
         self.rendezvous_point_for[payload.cookie] = circuit
 
-        self.send_cell([source_address], u"rendezvous-established",
+        self.send_cell(source_address,
                        RendezvousEstablishedPayload(circuit.circuit_id, payload.identifier, self.my_estimated_wan))
 
-    @tc_lazy_wrapper_unsigned(RendezvousEstablishedPayload)
+    @unpack_cell(RendezvousEstablishedPayload)
     def on_rendezvous_established(self, source_address, payload, circuit_id):
-        if not self.request_cache.has(u"establish-rendezvous", payload.identifier):
+        if not self.request_cache.has("establish-rendezvous", payload.identifier):
             self.logger.warning("Invalid rendezvous-established request identifier")
             return
 
-        rp = self.request_cache.pop(u"establish-rendezvous", payload.identifier).rp
+        rp = self.request_cache.pop("establish-rendezvous", payload.identifier).rp
 
         sock_addr = payload.rendezvous_point_addr
         rp.rp_info = (sock_addr[0], sock_addr[1], self.crypto.key_to_bin(rp.circuit.hops[-1].public_key))
