@@ -40,6 +40,11 @@ class MockRestIPv8(object):
     def get_overlay(self, overlay_cls):
         return next((o for o in self.overlays if isinstance(o, overlay_cls)), None)
 
+    def add_strategy(self, overlay, strategy, target_peers):
+        with self.overlay_lock:
+            self.overlays.append(overlay)
+            self.strategies.append((strategy, target_peers))
+
     def unload_overlay(self, instance):
         self.overlays = [overlay for overlay in self.overlays if overlay != instance]
         self.strategies = [(strategy, target_peers) for (strategy, target_peers) in self.strategies
@@ -94,7 +99,8 @@ class RESTTestBase(TestBase):
                         overlay.walk_to(other.endpoint.wan_address)
         await self.deliver_messages()
 
-    async def make_request(self, node, endpoint, request_type, arguments=None, json_response=True, json=None):
+    async def make_request(self, node, endpoint, request_type, arguments=None, json_response=True, json=None,
+                           expected_status=200):
         """
         Forward an HTTP request of the specified type to a url, with the specified set of arguments.
 
@@ -102,7 +108,10 @@ class RESTTestBase(TestBase):
         :param endpoint: the endpoint of this request (i.e. http://<interface>:<port>/<endpoint>)
         :param request_type: the type of request (GET, POST, PUT, DELETE, etc.)
         :param arguments: the arguments to be attached to the request. This should be a dictionary or None
-        :return: a directionary object with the response
+        :param json_response: whether the response is expected to be JSON
+        :param json: a JSON-serializable dictionary that is sent when making the request
+        :param expected_status: the status code returned in the response, defaults to 200
+        :return: a dictionary object with the response
         """
 
         # Using localhost in the URL will cause aiohttp to first try ::1, causing a 1s delay for each request
@@ -111,4 +120,6 @@ class RESTTestBase(TestBase):
 
         async with ClientSession() as session:
             async with session.request(request_type, url, json=json, params=arguments, headers=headers) as response:
+                self.assertEqual(response.status, expected_status,
+                                 "Expected HTTP status code %d, got %d" % (expected_status, response.status))
                 return await response.json(content_type=None) if json_response else await response.read()
