@@ -1,5 +1,6 @@
 import time
 from asyncio import Future, sleep
+from unittest.mock import Mock
 
 from .test_community import MockDHTProvider
 from ...base import TestBase
@@ -7,8 +8,8 @@ from ...mocking.exit_socket import MockTunnelExitSocket
 from ...mocking.ipv8 import MockIPv8
 from ....messaging.anonymization.community import CIRCUIT_TYPE_RP_DOWNLOADER, TunnelSettings
 from ....messaging.anonymization.hidden_services import HiddenTunnelCommunity
-from ....messaging.anonymization.tunnel import CIRCUIT_TYPE_DATA, CIRCUIT_TYPE_IP_SEEDER, IntroductionPoint,\
-    PEER_FLAG_EXIT_ANY, PEER_SOURCE_DHT
+from ....messaging.anonymization.tunnel import (CIRCUIT_TYPE_DATA, CIRCUIT_TYPE_IP_SEEDER, IntroductionPoint,
+                                                PEER_FLAG_EXIT_BT, PEER_FLAG_SPEED_TEST, PEER_SOURCE_DHT)
 from ....peer import Peer
 from ....util import fail, succeed
 
@@ -76,10 +77,8 @@ class TestHiddenServices(TestBase):
         settings = TunnelSettings()
         settings.min_circuits = 0
         settings.max_circuits = 0
+        settings.remove_tunnel_delay = 0
         ipv8 = MockIPv8(u"curve25519", HiddenTunnelCommunity, settings=settings)
-
-        ipv8.overlays = []
-        ipv8.strategies = []
         ipv8.overlay.ipv8 = ipv8
 
         # Then kill all automated circuit creation
@@ -100,8 +99,8 @@ class TestHiddenServices(TestBase):
 
         for node in self.nodes:
             exit_sockets = node.overlay.exit_sockets
-            for exit_socket in exit_sockets:
-                exit_sockets[exit_socket] = MockTunnelExitSocket(exit_sockets[exit_socket])
+            for circuit_id in exit_sockets:
+                exit_sockets[circuit_id] = MockTunnelExitSocket(exit_sockets[circuit_id])
 
     async def assign_exit_node(self, node_nr):
         """
@@ -109,7 +108,7 @@ class TestHiddenServices(TestBase):
         """
         exit_node = self.create_node()
         self.private_nodes.append(exit_node)
-        exit_node.overlay.settings.peer_flags.add(PEER_FLAG_EXIT_ANY)
+        exit_node.overlay.settings.peer_flags.add(PEER_FLAG_EXIT_BT)
         public_peer = Peer(exit_node.my_peer.public_key, exit_node.my_peer.address)
         self.nodes[node_nr].network.add_verified_peer(public_peer)
         self.nodes[node_nr].network.discover_services(public_peer, exit_node.overlay.master_peer.mid)
@@ -177,7 +176,7 @@ class TestHiddenServices(TestBase):
         data = b'PACKET'
         _, circuit = e2e_path[0]
         self.nodes[2].overlay.on_raw_data = lambda _, __, rdata: self.received_packets.append(rdata)
-        self.nodes[0].overlay.send_data([circuit.peer], circuit.circuit_id, ('0.0.0.0', 0), ('0.0.0.0', 0), data)
+        self.nodes[0].overlay.send_data(circuit.peer, circuit.circuit_id, ('0.0.0.0', 0), ('0.0.0.0', 0), data)
         await self.deliver_messages()
         self.assertEqual(len(self.received_packets), 1)
         self.assertEqual(self.received_packets[0], data)
@@ -289,3 +288,37 @@ class TestHiddenServices(TestBase):
         # Node 2 should be known as an introduction point
         peers = [ip.peer for ip in self.nodes[3].overlay.swarms[self.service].intro_points]
         self.assertCountEqual(peers, [self.nodes[1].my_peer, self.nodes[2].my_peer])
+
+    async def test_test_request_e2e(self):
+        """
+        Check if sending test-request messages over an e2e circuit works as expected.
+        """
+
+        future = Future()
+
+        self.nodes[0].overlay.join_swarm(self.service, 1, future.set_result, seeding=False)
+        self.nodes[2].overlay.join_swarm(self.service, 1, future.set_result)
+        self.nodes[2].overlay.settings.peer_flags.add(PEER_FLAG_SPEED_TEST)
+
+        await self.introduce_nodes()
+        await self.create_intro(2, self.service)
+        await self.assign_exit_node(0)
+
+        await self.nodes[0].overlay.do_peer_discovery()
+        await self.deliver_messages()
+
+        await future
+
+        send_cell = self.nodes[0].overlay.send_cell
+        self.nodes[0].overlay.send_cell = Mock(wraps=send_cell)
+        on_test_request = self.nodes[2].overlay.on_test_request
+        self.nodes[2].overlay.decode_map_private[30] = Mock(wraps=on_test_request)
+
+        circuit, = self.nodes[0].overlay.find_circuits(ctype=CIRCUIT_TYPE_RP_DOWNLOADER)
+        data, _ = await self.nodes[0].overlay.send_test_request(circuit, 3, 6)
+        self.assertEqual(len(self.nodes[0].overlay.send_cell.call_args[0][1].data), 3)
+        self.assertEqual(len(data), 6)
+        self.nodes[2].overlay.decode_map_private[30].assert_called_once()
+
+        self.nodes[0].overlay.leave_swarm(self.service)
+        self.nodes[2].overlay.leave_swarm(self.service)
