@@ -1,7 +1,6 @@
 from ..mocking.endpoint import AutoMockEndpoint
 from ...messaging.anonymization.tunnel import DataChecker, TunnelExitSocket
 from ...messaging.interfaces.endpoint import EndpointListener
-from ...util import succeed
 
 
 class MockTunnelExitSocket(TunnelExitSocket, EndpointListener):
@@ -9,6 +8,7 @@ class MockTunnelExitSocket(TunnelExitSocket, EndpointListener):
     def __init__(self, parent):
         self.endpoint = AutoMockEndpoint()
         self.endpoint.open()
+        self.parent = parent
 
         TunnelExitSocket.__init__(self, parent.circuit_id, parent.peer, parent.overlay)
         EndpointListener.__init__(self, self.endpoint, main_thread=False)
@@ -19,10 +19,10 @@ class MockTunnelExitSocket(TunnelExitSocket, EndpointListener):
         self.enabled = True
 
     def sendto(self, data, destination):
-        if DataChecker.is_allowed(data):
+        if DataChecker.could_be_bt(data) or DataChecker.could_be_ipv8(data):
             self.endpoint.send(destination, data)
         else:
-            raise AssertionError("Attempted to exit data which is not allowed" % repr(data))
+            raise AssertionError("Attempted to exit data which is not allowed: %s" % repr(data))
 
     def on_packet(self, packet):
         source_address, data = packet
@@ -30,4 +30,4 @@ class MockTunnelExitSocket(TunnelExitSocket, EndpointListener):
 
     async def close(self):
         await self.shutdown_task_manager()
-        return succeed(True)
+        await self.parent.close()
