@@ -48,3 +48,13 @@ def maybe_coroutine(func, *args, **kwargs):
     async def coro():
         return value
     return coro()
+
+
+def coroutine(func):
+    async def call_async(*args, **kwargs):
+        return func(*args, **kwargs)
+    return call_async
+
+
+def strip_sha1_padding(s: bytes):
+    return s[12:] if s.startswith(b'SHA-1\x00\x00\x00\x00\x00\x00\x00') else s
